@@ -232,6 +232,13 @@ LIBEXP void LipIntSetData( int id, int dim, int K, double* x, double* y, int tes
 
 /*--------------------------------------------------------*/
 /* interface to the members of SLipInt class */
+
+LIBEXP double	LipIntValues( int id, int *Dim, int *Ndata, double* x, double* Xd,double* y,  double* Lipconst, int* Index, int num, double* results)
+{ for (int i=0;i<num;i++) 	
+	results[i] = umsli.at( id)->Value(*Dim, *Ndata, x+ i* sizeof(double)* *Dim, Xd, y, *Lipconst, Index); 
+return results[num-1];
+}
+
 LIBEXP double	LipIntValue( int id, int *Dim, int *Ndata, double* x, double* Xd,double* y,  double* Lipconst, int* Index)
 { return umsli.at( id)->Value(*Dim, *Ndata, x, Xd, y, *Lipconst, Index); }
 
