@@ -25,6 +25,7 @@ functions:
 import numpy as np
 import math
 from  _liblip import ffi, lib as fm
+from memory_profiler import profile
 
 ###
 # Helper functions
@@ -62,6 +63,7 @@ def create_float_zeros_as_CFFI_double( n):
     px = ffi.cast( "double *", x.ctypes.data)
     return x, px
 
+@profile
 def convert_py_float_to_cffi( x):
     if x is not None:
         if isinstance( x, np.ndarray) and ( x.flags.c_contiguous == True) and ( x.dtype == np.float64):
@@ -76,6 +78,7 @@ def convert_py_float_to_cffi( x):
     return px, pxcffi
 
 
+@profile
 def convert_py_int_to_cffi( x):
     if x is not None:
         if isinstance( x, np.ndarray) and ( x.flags.c_contiguous == True) and ( x.dtype == np.int32):
@@ -360,7 +363,8 @@ def STCValue( x, id = 0):
 
 # New wrapper function returning a list of predictions 
 # This avoids multiple calls to np.ascontiguousarray()
-# Next step: to impeove performance implement the loop in C++ 
+# Next step: to improve performance implement the loop in C++ 
+@profile
 def LipIntValues( n, Dim, Ndata, x, Xd, y, Lipconst, Index, id = 0):
     trace( "double	LipIntValues(int n, int* Dim, int* Ndata, double* x, double* Xd,double* y,  double* Lipconst, int* Index)")
     pDimnp, pDim = convert_py_int_to_cffi( Dim)
@@ -370,11 +374,112 @@ def LipIntValues( n, Dim, Ndata, x, Xd, y, Lipconst, Index, id = 0):
     pynp, py = convert_py_float_to_cffi( y)
     pLipconstnp, pLipconst = convert_py_float_to_cffi( Lipconst)
     pIndexnp, pIndex = convert_py_int_to_cffi( Index)
-
-    pred = [ fm.LipIntValue( id, pDim, pNdata, px[i * Dim], pXd, py, pLipconst, pIndex) for i in range( 0, n)]
+    # pred = [ fm.LipIntValue( id, pDim, pNdata, px + ( i * Dim), pXd, py, pLipconst, pIndex) for i in range( 0, n)]    
+    # pred, ppred = create_float_zeros_as_CFFI_double( n)
+    # for i in range( n):
+    #     pred[i] = fm.LipIntValue( id, pDim, pNdata, px + ( i * Dim), pXd, py, pLipconst, pIndex)
+          
+    pred, ppred = create_float_zeros_as_CFFI_double( n)
+    fm.LipIntValues( id, pDim, pNdata, px, pXd, py, pLipconst, pIndex, n, ppred)   
+    
     return pred
 
 
+def	LipIntValuesAuto( n, Dim, Ndata, x,Xd, y, Index, num, id = 0):
+    trace( "double	LipIntValuesAuto(int n, int* Dim, int* Ndata, double* x,double* Xd, double* y, int* Index)")
+    pDimnp, pDim = convert_py_int_to_cffi( Dim)
+    pNdatanp, pNdata = convert_py_int_to_cffi( Ndata)
+    pxnp, px = convert_py_float_to_cffi( x)
+    pXdnp, pXd = convert_py_float_to_cffi( Xd)
+    pynp, py = convert_py_float_to_cffi( y)
+    pIndexnp, pIndex = convert_py_int_to_cffi( Index)
+    
+    pred, ppred = create_float_zeros_as_CFFI_double( n)
+    for i in range( n):
+        pred[i] = fm.LipIntValueAuto( id, pDim, pNdata, px + ( i * Dim), pXd, py, pIndex)
+    return pred
+
+def	LipIntValuesCons( n, Dim, Ndata, Cons, x, Xd,y,  Lipconst, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntValuesConsLeftRegion( n, Dim, Ndata, Cons, x, Xd,y,  Lipconst, Region, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntValuesConsRightRegion( n, Dim, Ndata, Cons, x, Xd,y,  Lipconst, Region, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntValuesLocal( n, Dim, Ndata, x, Xd,y, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntValuesLocalCons( n, Dim, Ndata,Cons, x, Xd,y, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntValuesLocalConsLeftRegion( n, Dim, Ndata,Cons, x, Xd,y, Region, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntValuesLocalConsRightRegion( n, Dim, Ndata,Cons, x, Xd,y, Region, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValues( n, Dim, Ndata, x, Xd,y,  Lipconst, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesAuto( n, Dim, Ndata, x,Xd, y, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesCons( n, Dim, Ndata, Cons, x, Xd,y,  LipIntInfVerifyMonotonicityLeftRegionLipconst, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesConsLeftRegion( n, Dim, Ndata, Cons, x, Xd,y,  Lipconst, Region, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesConsRightRegion( n, Dim, Ndata, Cons, x, Xd,y,  Lipconst, Region, Index, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesLocal( n, Dim, Ndata, x, Xd,y, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesLocalCons( n, Dim, Ndata,Cons, x, Xd,y, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesLocalConsLeftRegion( n, Dim, Ndata,Cons, x, Xd,y, Region, num, id = 0):
+	pred = 0 
+	return pred
+
+def	LipIntInfValuesLocalConsRightRegion( n, Dim, Ndata,Cons, x, Xd,y, Region, num, id = 0):
+	pred = 0 
+	return pred
+
+def STCValues( n, x, num, id = 0 ):
+	pred = 0 
+	return pred
+
+def STCValuesExplicit( n, x, num, id = 0 ):
+	pred = 0 
+	return pred
+
+def LipIntValuesExplicitDim( n, dim, x, num, id = 0):
+	pred = 0 
+	return pred
+
+def LipIntValuesDim( n, dim, x, num, id = 0):
+	pred = 0 
+	return pred
+
+
 ###
 # Remaining wrapper functions 
 ###
